;  ************************************************************************************
;  *   PCKB12.asm  -  works!  code designed for the Atmel AVR 90S1200  
;  *
;  *     version 1.50 	5/25/2004    Alan Probandt   Portland Oregon USA
;  *
;  *    v1.50  changed INT0 interrupt to falling edge only
;  *    v1.46  repositioned the F4 and F#4 keys and added G4 to Enter key
;  *	v1.45  repositioned the A#2 and B2 keys to lower left corner of keybrd
;  *    v1.44  added Enter key to Note Map as F#4
;  *	v1.43  added voice mapping to the function keys Fn1 to Fn8.  Fn9 to Fn12 map voice #9 - 12
;  *	v1.42  removed DIP switches and MIDI transmit modes, changed filename, now PCKB to MIDI only
;  *	v1.41  added keypad digit decode for voice number selection
;  *	v1.40  Added Sustain on/off for CapsLock toggle
;  *
;  *	Differences from AVR2313 code for the same program:
;  *	-Keypress decoded by stepping through EEPROM table to find match
;  *	-Stack usage minimized
;  *	-Limited choices for extended scancode keys
;  *
;  *  [ Use Terminal font in text editor to format this file correctly ]
;  *
;  *
;  *   PC keyboard clock signal on AVR pin 6 (INT1)
;  *   PC keyboard data signal on AVR pin 7 (PORTD bit 3)
;				
;    					Atmel AVR 90S1200
;  		 active low	PIN 1 RESET	| VCC  PIN 20  +5V
;   				PIN 2 PD0	| PB7  PIN 19  SCK  in-circuit programming
;   				PIN 3 PD1 	| PB6  PIN 18  MISO in-circuit programming
;   	use only    PIN 4 XTAL1	| PB5  PIN 17  MOSI in-circuit programming
;    	3.579MHz	PIN 5 XTAL2	| PB4  PIN 16
;  	INT0 kbd clock  PIN 6 PD2 	| PB3  PIN 15
;   	kbd data 	PIN 7 PD3 	| PB2  PIN 14
;   	MIDI out	PIN 8 PD4  	| PB1  PIN 13
;   		 		PIN 9 PD5  	| PB0  PIN 12
;   				PIN 10 gnd	| PD6  PIN 11
;    
;  ****************************************************************************************************
;    How the program works
;
;	The program maps keypresses from a PS2 keyboard to MIDI note values.  The notes B1 to F#4 have MIDI 
;	note numbers 33 to 79, 46 notes total.  The EEPROM table has 46 entries for notes. 	When a key is
;	pressed on the PS2 keyboard, it sends a scancode number for the key to the AVR.  The scancodes are
;	listed in the table according to the key's assigned MIDI note value.  For example, C2 is 13 steps 
;	from B1 so the scancode for the key that will sound C2 is in the 13th (from 0) position of the table.
;	At a keypress, the code starts from the table beginning and compares each successive value to the 
;	scancode. If found, the offset into the table is added to the Octave Offset (33) to get the MIDI
;	note value.  This is sent to the synthesizer as a MIDI message.
;
;  	The code first checks if the MIDI note in 'DecodeValue' is already playing by checking for its MIDI 
;	note number in each of the eight storage registers R8-R15.  If the note is already in one of the 
;	storage registers, then the user has held the typewriter key down long enough to trigger the auto-repeat.  
;	In this case, the repeated keypress is ignored. The note continues to play uninterrupted.  There doesn't 
;	seem to be any way to turn off the auto-repeat of the PC keyboard.
; 	If the note number from 'DecodeValue' is not found in the storage register set, the set is checked to 
;	see if there is space for a new note.  The code is set to allow eight notes to be played at once.  
;	Each note number is stored in one of the eight registers.  If a register is not holding a note number 
;	(a note which is currently sounding by the synth), then the register holds the value 0xFFh.  The code 
;	searches the storage registers for the first one that holds 0xFFh.  If there is none, then eight notes 
;	are sounding and the code exits.  If a register is found with 0xFFh, the value from 'DecodeValue' is 
;	copied into that register and the note value is sent to synthesizer as a Note-On.
;  	When a key is released, its corresponding note value is found in the register set (r8-r15).  The register
;	is set to 0xFFh and a MIDI note-on message with velocity set to zero is sent to the synth for that 
;	note number.  The synth releases the note.
;  	This technique allows individual keys to play without affecting the other keys that are being held down.


.nolist
.include "1200def.inc"
.list

.equ	MIDItxpin			= 4	 ; MIDI Transmit pin
.equ	MIDItxPORT			= PORTD
.equ	DIPswitchPort		= PINB
.equ	INITIAL_BITCOUNT 	= 11   ;  used by Get_Scan
.equ	PARITY_BITCOUNT 	= 3    ;  used by Get_Scan
.equ	NOTEOFFSET			= 33  
.equ	CHANGE_VOICE_OFFSET = $30
.equ	SPACEBAR_OFFSET		= $2f
.equ	REGISTER_AVAILABLE	= $FF
.equ	POSSIBLE_NOTES 		= 8
.equ	FIRST_STORAGE_REG 	= 8
.equ	SUSTAIN_ON			= 127
.equ	SUSTAIN_OFF			= 0
.equ	NOTE_ON_VELOCITY  	= 0x50
.equ	NOTE_OFF_VELOCITY  	= 0
.equ	NOTE_ON_MSG 		= $90
.equ	CHANGE_VOICE_MSG 	= $c0
.equ	MIDI_CONTROLLER_MSG = $b0 
.equ	MIDI_HOLD1			= $40
.equ	ALL_NOTES_OFF_BYTE 	= 123 ; $7b
.equ	ALL_NOTES_OFF_2nd 	= 0
.equ	CapsToggleValue		= 0b01000000
.equ  	KBPort				= PinD;KBport,KBdataPin 
.equ	KBDataPin			= 3
.equ	FIRST_VXMap_REG		= 22

.equ	PSEUDO_STACK 			= $65 ; imitate a real stack when using ICE200 for 1200 chip
.equ	PSEUDO_STACK_POINTER 	= $3d

.equ	KBD_ACK					= $0fa
.equ	T_flag 					= 6
.equ	Sign_flag				= 4
.equ	SM_flag					= 4 ; Sleep Mode (unused) - delay loop counter bit flag { 0 = long period is timed out}
.equ 	PushSwitch1  			= 4

; keyflag register's bit values
.equ	Edge_flag	= 7 ; use up or down edge while inputting char from keyboard
.equ	Caps_flag	= 6 ; Caps Lock pressed on -- no NoteOff transmitted
.equ	Alt_flag	= 5 ; left Alt key is being pressed
.equ	Break_flag	= 4	; the last scancode was an $f0 - the break code sent when a key is released
.equ	OnOff_flag	= 3	; set means turn on the note whose MIDI number was decoded from the most recent scancode
.equ	E0_flag		= 1	; the extended scancode $E0 was sent - use the E0table to decode scancode


;*********************************************************************************************
;  Register definitions
;
;  lower registers
; r0 is used for EEPROM addressing
.def   	CurrentVoiceNumber = r1 ; selected voice on synth - used for incrementing and decrementing voice
.def	SustainValue	= r2    ; temporary register used to change HOLD1 message value
.def	NoteOffsetValue = r3    ; holds current octave offset value -value stored between keypresses
.def	KPvalue			= r4   	; used to decode keypad digit presses-value stored between keypresses
.def  	txbitcnt		= r5	; used by PutMIDI   
.def	DecodeValue		= r6  	; PC keyboard char decoded to NoteOn value, ChangeVoice value, or AllNotesOff cmd

;  upper registers
.def	bitcount		= r16	; used by INT0 irq  [getscan]	
.def  	scancode		= r17	; storage of data bits received from PC keyboard 
.def	scanbits		= r18 	; shift register for holding PC keyboard data while being assembled
.def	keyflags		= r19 
.def	temp			= r20	; scratch register
.def	irqtemp			= r21  	; interrupt's scratch register
.def	PMtemp			= r31	; PutMIDI temp register counter
.def	EEPROM_Ptr 		= r31   ; temp reg used by DecodeData to find scancode in EEPROM table
.def	KPtemp			= r31	; temporary register used to decode keypad digit presses only
.def	regcnt			= r31   ; temp counter used  in PlayNote, TurnOffNote, ChangeVoice, Mode One setup    
.def	CVtemp			= r31	; temp counter for ChangeVoice


.cseg
.org   $0000
		rjmp		reset		; Reset handler $000
.org  INT0addr          	
		rjmp		get_scan_dn	; External interrupt 0 handler
.org  OVF0addr          	
        reti					; Timer0 overflow handler 
.org  ACIaddr        
       	reti					; Analog comparator handler 

RESET:
;		ldi			temp,PSEUDO_STACK ; imitate a real stack when using ICE200 for 1200 chip
;		out			PSEUDO_STACK_POINTER,temp	;init Stack Pointer - does nothing on real 1200 chip
        	
; set-up ports for input/output and pull-up resistors
		ser			temp
		out			portb,temp	; port B pull-up resistors are all on
		out			portd,temp	; port D pull-up resistors are all on 
		clr			temp
		out			ddrb,temp  ; port b is all inputs  the DIP switches are here Pb0 to PB3
		ldi			temp,(1<<MIDItxpin)
		out			ddrd,temp  ; port b is all inputs  the DIP switches are Pb0 to PB3 - pushswitch is PB4
           	
;*******************************************************************************************
;
;   Get keypresses from standalone PC keyboard - convert to notes and send to MIDI port
;
;   Subroutines:	'decode_data'
;
;	Interrupt procedures:	external interrupt int0 -- 'getscan'
;
;   Most time spent in 'GetChar' which checks T flag for fully received keypress from PC keybrd
;   When T is set, 'decode_data' called to convert keypress to note or cmd.
;*******************************************************************************************
		ldi			temp,(1<<ISC01) ; setup INT0 interrupt on falling edge
		out			MCUCR, temp  	
		ldi			temp, (1<<INT0)
		out			GIMSK,temp      ;  enable external INT0 - General Interrupt Mask Register
		ldi			zl, FIRST_STORAGE_REG - 1  ; fill note storage area with REGISTER_AVAILABLE sentinels
		ldi			regcnt, POSSIBLE_NOTES
		ldi			temp, REGISTER_AVAILABLE	
rst1:	inc			zl
		st			z, temp
		dec			regcnt
		brne		rst1  
		
;  Reset places the values zero to seven in the VxMap register set (r22 to R29) 
		ldi			zl, FIRST_VXMap_REG - 1  ; fill note storage area with REGISTER_AVAILABLE sentinels
		ldi			regcnt, POSSIBLE_NOTES
		clr			temp
rst2:	inc			zl
		st			z, temp
		inc			temp
		dec			regcnt
		brne		rst2  
           	
	    clr			KPvalue	; used to decode keypad digit presses
		clr			keyflags ;  edge = bit 7:    shift = bit 1:   Break = bit 0
		ldi			bitcount,INITIAL_BITCOUNT ; = 11
		clt			;   T flag used to indicate that a char from kybd is ready to transmit
		ldi			temp, NOTEOFFSET
		mov			NoteOffsetValue, temp
		clr			CurrentVoiceNumber
		sei			; switch on the general interrupt enabler
	       
;*****************************************************************************
;
;  Main Loop of the Program  99.9% of time spent here
;
;*****************************************************************************	
      
GetChar:	
		in			temp, SREG 
		sbrs		temp,T_flag   ; if T flag (SREG bit 6) is set, then char from keyboard is ready to transmit
		rjmp		GetChar
		
		clt			 ; clear T flag - prepare for next char
		rcall		decode_data  ; convert the scancode from the keyboard into an ASCII character
								; send scancode and keyflags - return DecodeValue
		tst			DecodeValue    ; if DecodeValue =0, then ignore scancode - it's a shift character or break scancode
		brne		Valid_key     
		rjmp		GetChar

Valid_key:
		ldi			temp,SPACEBAR_OFFSET  ; $2e is this program's code number for All_Notes_Off
		cp			DecodeValue, temp
		breq		AllNotesOff
		brlo		PlayNote  ; DecodeValue is lower in the keypress decode table than the spacebar
		brbc		Sign_flag, ChangeVoice   ; br if sign reg bit set - DecodeValue's table offset is greater the spacebar
 	
PlayNote:  
		sbrc		keyflags,OnOff_flag   ; if OnOff flag is set, then char is a MIDI note number to be played
		rjmp		playMIDInote
		sbrs		keyflags,OnOff_flag  ; if OnOff flag is clear, then char is a playing MIDI note to be turned off
		rjmp		TurnOffMIDInote

; *********************************************************************************************
;  AllNotesOff 	1) 	Put Note_Available sentinal into the register bank for all possible notes.
;					All possible note spaces are available because all notes are off.
;				2)	Send the All_Notes_Off  MIDI control message to the synthesizer.
;
;				3)  jump back to GetChar routine
;	When the Hold1 is enabled (CapsLock key) then AllNotesOff doesn't work.
;*********************************************************************************************
AllNotesOff:  

		ldi			ZL,FIRST_STORAGE_REG - 1
		ldi			regcnt,POSSIBLE_NOTES
		ser			temp
ANO1:	inc			zl  ; first check 8 storage registers to see if the note is already playing
		st			z,temp
		dec			regcnt
		brne		ANO1 ; make all storage registers available for new notes at space-bar press  
		
		sbrc		keyflags, Caps_flag    ; check if Caps_flag is clear
		rjmp		ANO3	; if yes then send AllNotesOff message and exit

ANO2:	ldi			Temp, MIDI_CONTROLLER_MSG      ; send AllNotesOff message 
		rcall		putMIDI
		ldi			Temp, ALL_NOTES_OFF_BYTE 
		rcall		putMIDI
		ldi			Temp, ALL_NOTES_OFF_2nd
		rcall		putMIDI
  		
		sbrs		keyflags, Caps_flag ; check if Caps_flag is set
		rjmp		GetChar       ; if not then exit       
		ldi			temp, MIDI_CONTROLLER_MSG ; turn on HOLD1
        rcall		putMIDI                     
        ldi			temp, MIDI_HOLD1
        rcall		putMIDI                     
        ser			temp
        rcall		putMIDI
        
        rjmp		GetChar
        
ANO3:	ldi			temp, MIDI_CONTROLLER_MSG ; turn off HOLD1 if Caps_flag is set
        rcall		putMIDI                     
        ldi			Temp, MIDI_HOLD1
        rcall		putMIDI                     
        clr			temp 
        rcall		putMIDI 
        rjmp		ANO2
	           	
; **********************************************************************************************************
;  Function key was pressed - Decode Value has the Fn key number - 1
;  Reset places the values zero to seven in the VxMap register set (r22 to R29).
;
;  If a function key is pressed, check if the left Alt key is being held down, (the Alt_flag will be set).
;  	If the Alt key is down, check if the function key is Fn1 to Fn8.
;  	  Fn1 to Fn8 yes, then put the current voice number into the selected function key's voice map register.
;	  Fn1 to Fn8 no, then send a Program Change Message 7 through 11 to the synth (depending on the Fn key).
;	If the Alt key is not being pressed, get the value in the selected function key's voice map register
;	  and send to the synth as a Program Change Message value.
;	 
;  
;  ChangeVoice 	1) 	Put Note_Available sentinel into the register bank for all possible notes.
;		   			This indicates that all possible note spaces are available because all notes are off.
;				2)  Send the MIDI program change message to the synthesizer.  This will turn off all
;					sounding notes for the selected MIDI channel (channel 1 by default).
;				3)  Jump back to GetChar routine.
;***********************************************************************************************************	            	
ChangeVoice:
		mov 		temp,DecodeValue                                                                    
		subi		temp,CHANGE_VOICE_OFFSET ;  remove the EEPROM table offset for function key decoding
				
		cpi			temp, 8   ; only eight registers available for voice number mapping
		brlo		CV0
		
		; function key Fn8 or greater pressed, just send voice number 8 to 12 to the synth
		mov			CurrentVoiceNumber,temp ; temp has # of Fn key pressed (0x07=Fn8 ... 0x0b=Fn12)
		rjmp		DoProgramChange
		         
CV0:	sbrc		keyflags, Alt_flag ; check Alt key     
		rjmp		AltPressed
		
; Alt key not pressed (Alt_flag is clear), so get value from VxMap reg and send to synth as a Program Change
		ldi			ZL,FIRST_VXMap_REG	; r22
		add			ZL, temp 			; temp = 0 through 7
		ld			temp, z
		mov			CurrentVoiceNumber,temp ; temp now has voice number found in VxMap register for Fn key pressed
		rjmp		DoProgramChange
		
AltPressed:	 
		ldi			ZL,FIRST_VXMap_REG  ; r22
		add			ZL, temp        	; temp has 0 to 7 (for Fn1 to Fn8)
		st			z, CurrentVoiceNumber
		rjmp		GetChar  ; don't change voice number if Alt key is pressed, just map Current Voice Num to Fn key
		  		
DoProgramChange:
		ldi			ZL,FIRST_STORAGE_REG - 1  ; clear the note storage regs because changing voice turns off all notes
		ldi			regcnt,POSSIBLE_NOTES
		ser			temp
CV1: 	inc			zl  ;a change voice command turns off all sounding notes
		st			z,temp
		dec			regcnt
		brne		CV1 ; make all storage registers available for new notes when voice changes	

		ldi			temp,CHANGE_VOICE_MSG 
		rcall		putMIDI
		mov			temp,CurrentVoiceNumber
		rcall		putMIDI
		rjmp  		GetChar
	
;********************************************************************************
; GET_SCAN:  ; signal handler for external interrupt int0 (AVR pin 6)
;   called by each transition of the keyboard clock signal
;   assemble the scancode from keyboard data signal
; (  UltraEdit32 - use column mode or terminal font to see correctly)
;clk---|_|--|_|--|_|--|_|--|_|--|_|--|_|--|_|--|_|--|_|--|_|----------   (---- = logic high +5v)
;data-\___/-----\___/-----\___/-----\___/-----\___/--------------------
;       11   10   9    8    7    6    5    4    3    2    1 bitcount on fall edge
;     start   0   1    2    3    4    5    6    7    P   stop   data bit
;  Bitcnt 3 to 10 is data. start bit = 11, parity bit=bit 2;  stop bit = bit 1
;*********************************************************************************
;get_scan: ;    Routine first entered at falling edge; then on both edges
;
;		sbrc		keyflags,Edge_flag  ; test edge flag
;		rjmp		up_edge  ; bit 7 (edge) = 0 => falling  ;; bit 7 (edge) = ff => rising
;		cpi 		bitcount,INITIAL_BITCOUNT ; falling edge of the FIRST clock pulse from the PC keyboard ; start bit
;		breq		I1two
;		cpi			bitcount,PARITY_BITCOUNT ; = 3  ; test for parity bit and stop bit
;		brlo		I1two  ; must use bitcount 3 for compare because branch tests only for lower
;		lsr			scanbits   ; shift data right one bit - data bit 7 gets 0
;		sbic		KBport,KBdataPin  ;set scancode bit if keyboard data bit is set
;		ori			scanbits,$80 ; if data from kbrd is 1, then set bit 7 only and let other bits unchanged
;		ldi 		irqtemp, ( (1<<ISC01) | (1<<ISC00) )
;		out 		MCUCR, irqtemp  ; Set interrupt on rising edge - INT0
;		sbr			keyflags,(1<<edge_flag) ; set edge flag
;		reti	    	
;I1two:	ldi 		irqtemp, ( (1<<ISC01) | (1<<ISC00) ) ;  just exit if bitcnt is 11, 2, or 1
;		out			MCUCR, irqtemp ; Set interrupt on rising clock edge - INT0
;		sbr			keyflags,(1<< Edge_flag)  ; set edge flag
;		reti    		
;up_edge:	
;		ldi 		irqtemp, 1<<ISC01
;		out			MCUCR, irqtemp ; Set interrupt on falling clock edge - INT0
;		cbr			keyflags,1<<Edge_flag  ; clear edge flag
;		dec 		bitcount
;		brne		exit_int1	; All bits received?
;		ldi			bitcount,INITIAL_BITCOUNT
;		set			; set T flag to let main program know a new char is ready
;		mov			scancode,scanbits ; scanbits can be used to assemble new char while scancode is being decoded
;exit_int1:  	  ; clear any pending interrrupt
;		clr			irqtemp
;		out			GIMSK,irqtemp      ;  disable external INT0 - General Interrupt Mask Register		
;		out			MCUCR,irqtemp
;		ldi			irqtemp, (1<<ISC01) ; interrupt on falling edge
;		out			MCUCR,irqtemp
;		ldi			irqtemp, (1<<INT0)
;		out			GIMSK,irqtemp      ;  enable external INT0 - General Interrupt Mask Register
;		reti       	
;

;*****************************************************************************************************************
;  DECODE_DATA  -- subroutine to convert the scancode from the keyboard
;   input: scancode (register value)   output = DecodeValue (register value)
;
;   A key pressed down will send the one-byte scancode to the PC for most characters.
;   When the key is released, the keyboard sends an $f0 byte followed by the scancode of the key being released.
;   This routine uses a flag called Break to determine if the scancode comes from a keypress down or release.
;	Break == clear means that scancode just received is a keypress down and should be decoded to Note On.
;	Break == set means that the scancode just received was $f0 and the next scancode means turn Note Off.
;
; 	Shift keys are ignored
;*****************************************************************************************************************
decode_data:
	
;   first check that Break flag is clear. if yes, then keypress down received
		sbrc		keyflags, Break_flag   ; the previous scancode was $f0 when break_flag is set => key released
		rjmp		Break_set

;   Break is clear, so do check the first three special conditions	
		cpi			scancode,$f0  ; 	$F0 =Breakcode  finger-off-the-key identifier
		brne		notF0
		sbr			keyflags, (1<<Break_flag)  ; set Break flag  so that the next scancode will not be seen a keypress down	
		rjmp		setZero
		
notF0:  cpi			scancode, $11 ; left Alt key is being pressed down. Alt_flag gets set.  Next key should be Fn1 to Fn8.
		brne		isCapsLock
		sbr			keyflags, (1<<Alt_flag)
		rjmp		setZero
		
; CapsLock key pressed -- Code to turn off/on Sustain Petal			
isCapsLock:  
		cpi			scancode, $58  ; caps lock key
		brne		isExtendedCode
		ldi			temp, CapsToggleValue
		eor			keyflags, temp  ; toggle the value in keyflags reg for sustain
 		nop			; make sure the that the register gets updated value before reading it
		sbrc		keyflags, Caps_flag
		rjmp		TurnOnSustain  ; flag toggled to set so initialize note sustain
		clr			SustainValue ; turn off sustain petal
		rjmp		Sustain
TurnOnSustain:	
		ldi			temp, SUSTAIN_ON
		mov			SustainValue,temp
Sustain:
		ldi			Temp, MIDI_CONTROLLER_MSG
		rcall		PutMIDI
		ldi			Temp, MIDI_HOLD1
		rcall		PutMIDI
		mov			Temp, SustainValue
		rcall		PutMIDI
		rjmp 		setZero
		
isExtendedCode: ; check for E0 flag
		cpi			scancode,$e0  ;  $e0  extended char table
		brne		isE0set  ; scancode is not one of the special cases
		sbr			keyflags,(1<<E0_flag) ; set E0 flag			
		rjmp		setZero	
	
isE0set: ; if the E0_flag is set then E0 (extended char scancode) was the last previously sent
		sbrc		keyflags,E0_flag 	; test extended char flag 
		rjmp		get_E0char		; determine the action to take for the extended keypress 

; ************ insert other checks here
		cpi			scancode, $70  ; keypad 0
		brne		digit1
		ldi			temp, 0
		breq		DoDigit
digit1:	cpi			scancode, $69  ; keypad 1
		brne		digit2
		ldi			temp, 1
 		breq		DoDigit                        
digit2:	cpi			scancode, $72  ; keypad 2
		brne		digit3
		ldi			temp, 2
 		breq		DoDigit                        
digit3:	cpi			scancode, $7a  ; keypad 3
		brne		digit4
		ldi			temp, 3
		breq		DoDigit                        
digit4:	cpi			scancode, $6b  ; keypad 4
		brne		digit5
		ldi			temp, 4
		breq		DoDigit                        
digit5:	cpi			scancode, $73  ; keypad 5
		brne		digit6
		ldi			temp, 5
		breq		DoDigit                        
digit6:	cpi			scancode, $74  ; keypad 6 
		brne		digit7
		ldi			temp, 6
 		breq		DoDigit                        
digit7:	cpi			scancode, $6c  ; keypad 7 
		brne		digit8
		ldi			temp, 7
		breq		DoDigit                        
digit8:	cpi			scancode, $75  ; keypad 8
		brne		digit9
		ldi			temp, 8
 		breq		DoDigit                        
digit9:	cpi			scancode, $7d  ; keypad 9
		brne		TableLookUp
		ldi			temp, 9
 		breq		DoDigit
		
; if a keypad digit is pressed then check if it is the first or second digit pressed. 
; the first digit will have KPValue register equal to zero. 2nd digit will have bit 7 set and 
; first digit press (the ten's column value in bits 3 to 0).  
; Code arrives here with digit value of keypad press in Temp register.
DoDigit: tst		KPvalue  
		brne		KP1digit
		sbr			temp, 0b10000000  ; the highest bit acts as a flag for 1st or 2nd digit		
		mov			KPvalue,temp
 		rjmp		SetZero
		
KP1digit:
		mov			KPtemp,KPvalue   ; received 2nd digit, so multiply 1st digit by 10 and add 2nd
		cbr			KPtemp,0b10000000 ; remove 2nd digit flag (bit 7 of KPvalue acts as 2nd digit flag)
		mov			KPvalue,KPtemp	; KPvalue now holds the first digit value in bits 3-0
		lsl			KPvalue			; First digit multiply by ten 
		lsl			KPvalue			; by multiplying by 2 three times and adding original value twice
		lsl			KPvalue			; bit 7 to carry, bit 0 cleared
		add			KPvalue, KPtemp
		add			KPvalue, KPtemp
		add			KPvalue, temp		;  max value 99 decimal in KPvalue
		mov			CurrentVoiceNumber, KPvalue
		clr			KPvalue
		rcall		DoProgramChange
		rjmp		SetZero                     
 
;not shift, $E0, or $F0, and no flags are set;  so do a table lookup from the EEPROM section 
TableLookUp:
		clr 		EEPROM_Ptr
cmpnext: 
		out 		EEAR,EEPROM_Ptr	;Set the EEPROM's address
		sbi 		EECR,EERE       ;Send the Read strobe
		in			r0,EEDR         ;Put the data in the transmit register
		
		tst 		r0              ; reached the end of the table?
		breq 		setZero
		cp			r0,scancode
		breq 		scan_found
		
		inc 		EEPROM_Ptr
		rjmp 		cmpnext
scan_found:
		mov 		DecodeValue,EEPROM_Ptr
		sbr			keyflags,(1<<OnOff_flag)  ; set the note on/off flag		
		rjmp 		exit_decode
	
;  Break flag is set - this means the previous scancode was $f0 (a key release)
;   This scancode is either the Extended sentinel ($e0) or the number of the key being 
;	released. If the scancode is in the EEPROM table, then turn OFF the MIDI note.
;   If the released key has an extended code, then the clear the break and the 
;	extended flags, set DecodeValue to zero and exit.
Break_set:
		cbr			keyflags,(1<<Break_flag) ; clear Break flag
		cpi			scancode, $11	; left Alt flag is released
		brne		isE0
		cbr			keyflags, (1<<Alt_flag)
		rjmp		setZero
		
; If scancode is E0, then set the Break flag so that the next scancode is recognized 
; as the last of an extended release sequence instead of a normal keypress down.		
isE0:	cpi			scancode, $e0	; extended key press sentinel
		brne		isE0flagOn
		sbr			keyflags,(1<<Break_flag)
		sbr			keyflags,(1<<E0_flag) ; set extended flag		
		rjmp		setZero
	
; if the E0flag is on then this scancode is the extended character number that is being released.  Just clear flags.		
isE0flagOn:		
		sbrs		keyflags, E0_flag   ; Extended flag on?
		rjmp		TableLookUp1		;no, a note key is being released.  Find note and send MIDI note off message.
		cbr			keyflags,(1<<Break_flag) ; yes, clear Break flag
		cbr			keyflags,(1<<E0_flag) ; clear extended flag
		rjmp		setZero	
		
TableLookUp1:
		cbr			keyflags,(1<<OnOff_flag)  ; clear the note on/off flag
;determine which note needs to be turned off
       	clr 		EEPROM_Ptr  ; start scanning from the beginning of the EEPROM table
cmpnxt1: 
		out 		EEAR,EEPROM_Ptr ;Set the EEPROM's address  does 00 not work?
		sbi 		EECR,EERE        ;Send the Read strobe
		in			r0,EEDR           ;Put the data in the transmit register
		; will not execute next instruction until EEPROM is finished reading
		tst 		r0               ; reached the end of the table?
		breq 		setZero      ; not found so turn all off
		cp			r0,scancode
		breq 		scan_fnd1
		
		inc 		EEPROM_Ptr
		rjmp 		cmpnxt1
scan_fnd1:
		mov 		DecodeValue,EEPROM_Ptr
		rjmp		exit_decode
setZero:  
		clr			DecodeValue	; When Decode sub returns zero in DecodeValue reg, the main program ignores the scancode.
exit_decode: 		; subroutine's single exit point
		ret

;  the previous scancode was E0, and few extended keys are handled by the AVR1200 code.
;  The ones that are include the up/down arrow keys for changing octave range and the
;  left/right arrow keys for changing to the previous/next voice number.
get_E0char:
		cbr			keyflags,(1<<E0_flag)  ; clear the E0 flag
		cpi 		scancode,$72  	; up arrow - raise the note value of all the keys by one octave.
		brne		isDownArrow
		mov			temp,NoteOffsetValue
		cpi			temp,$7f-12
		brsh		setZero 		; branch is same or higher (unsigned)
		subi		temp,-12		; 'addi' in AVR is done by subtracting a negative number 
		mov			NoteOffsetValue,temp
		rjmp		setZero
	
isDownArrow:
		cpi 		scancode,$75	; down arrow - lower the note value of all the keys by one octave
		brne		isLeftArrow
		mov			temp,NoteOffsetValue; can't send a MIDI note on note that is less than zero
		cpi			temp,12  		; so check that NoteOffset value is at least 12 before subtracting 12
		brlo		setZero  		; branch is same or higher (unsigned)
		subi		temp,12			; lower the note value of all the keys by one octave
		mov			NoteOffsetValue,temp
		rjmp		setZero
		
isLeftArrow:
		cpi 		scancode,$6b	; left arrow - change voice to previous number in synth voice bank
		brne		isRightArrow	
		mov			temp, CurrentVoiceNumber
		dec			temp
		andi		temp, 0b01111111
		mov			CurrentVoiceNumber,temp
		rjmp		DoProgramChange
		
isRightArrow:
		cpi 		scancode,$74	; right arrow  - change voice to next number in synth voice bank
		brne		setZero		
		mov			temp, CurrentVoiceNumber
		inc			temp
		andi		temp, 0b01111111
		mov			CurrentVoiceNumber,temp
		rjmp		DoProgramChange	  ; do lots of jumping to minimize stack usage on AVR1200

					
; ***** end of DecodeData subroutine *******************

;.include "pckb3aux.asm" ; split the code file here when using WAVRASM.exe Atmel Windows AVR assembler.
; For WAVRASM, put all the following code into the file "pckb3aux.asm", delete from this file,
;   and uncomment the above INCLUDE statement.  Truly a nitwit situation!  C'mon, Atmel, fix this!

;*******************************************************************
;  playMIDInote    not a subroutine
;   receives: keyflags, DecodeValue 
;*******************************************************************
playMIDInote:
		ldi			ZL,FIRST_STORAGE_REG - 1
		ldi			regcnt,POSSIBLE_NOTES
PMN1:	inc			zl  ; first check 8 storage registers to see if the note is already playing
		ld			temp,z
		cp			temp, DecodeValue
		breq		PMN3 ; found identical note number, which means that the Note had been already playing.
;						The key was held down long enough trigger typematic auto-repeat.
		dec			regcnt
		breq		NewNote
		rjmp		PMN1
				
NewNote: 	ldi		ZL,FIRST_STORAGE_REG - 1
		ldi			regcnt,POSSIBLE_NOTES
PMN2:	inc			zl  
		dec			regcnt
		breq		PMN3
		ld			temp,z ; now check the 8 storage registers to see if there is a voice available 
		cpi			temp,REGISTER_AVAILABLE  ; to play the new note
		brne		PMN2
		st			z,DecodeValue
		
		ldi			Temp,NOTE_ON_MSG 
		rcall		putMIDI
		add			DecodeValue,NoteOffsetValue
		mov			Temp,DecodeValue
		rcall		putMIDI
		ldi			Temp,NOTE_ON_VELOCITY
		rcall		putMIDI        
		rjmp		GetChar	
		
PMN3: ; found identical note number  
;	key held down long enough trigger typematic auto-repeat
	
PMN_OutOfRegs:
		rjmp		GetChar

;*********************************************************
;. TurnOffMIDInote    not a subroutine
;   receives: keyflags, DecodeValue 
;   registers used:  keyflags, regcnt
TurnOffMIDInote:
		ldi			ZL,FIRST_STORAGE_REG - 1
		ldi			regcnt,POSSIBLE_NOTES
ToMN1:          	
		inc			zl
                	
		ld			temp,z	
		cp			temp,DecodeValue
		breq		ToMN2
		        	
		dec			regcnt
		breq		ToMN_OutOfRegs
			
		rjmp		ToMN1
	
ToMN2: ; found note number in register storage
		ser			temp
		st			z,temp  ; let program that this reg is available 
		ldi			Temp,NOTE_ON_MSG
		rcall		putMIDI
		add			DecodeValue,NoteOffsetValue	
		mov			Temp,DecodeValue
		rcall		putMIDI
		ldi			Temp,NOTE_OFF_VELOCITY
		rcall		putMIDI
		rjmp		getchar
		
ToMN_OutOfRegs:	rjmp		AllNotesOff  ; couldn't find note in storage so turn off all notes
	
	
;***************************************************************************
;*
;* "putMIDI"   Data bits are fed to an output using a timing loop, not an interrupt.
;* This is a subroutine.  It returns to the code that called it after the data 
;* is finished transmitting (after about 325 microseconds).
;*
;* This subroutine transmits the byte stored in the "temp" register
;*
;* Number of words	:14 including return
;* Number of cycles	:Depends on bit rate
;* Low registers used	:txbitcnt
;* High registers used	:PMtemp
;* Pointers used	:None
;*
;*  time used  == 10 * halfbit_delay + @120 cycles
;*  subroutine does not exit until finished
;
;delay--------|---|---|---|----|----|---|----|---|---|------
;data-----\___/---\___/----\___/----\___/----\___/-----------
;Txbit      10 9 9 8 8 7  7 6 6 5  5 4 4 3  3 2 2 1 1 0
;Tx       start  0   1    2    3    4    5    6   7   stop  
.equ	b	= 36	; ((b-1)*3) +10 = 116 cycles = 32uSec [clock period = 279.4nS]
;***************************************************************************

putMIDI:	
		ldi			PMtemp,10	; 1+8+sb (sb is # of stop bits) 
		mov			txbitcnt, PMtemp
		com			temp		; Invert everything [the lsr always puts a 0 into the carry]
		sec				; Start bit
putMIDI0:	
		brcc		putMIDI1		; 1 or 2
		cbi			MIDItxPORT,MIDItxpin	; 2   send a '0' (clear bit I/O)
		rjmp		putMIDI2		; 2   else	
putMIDI1:	sbi		MIDItxPORT,MIDItxpin	; 2   send a '1'
		nop				; 1  	
putMIDI2:	ldi		PMtemp,b 		; 1 cycle
bit_delay1: dec		PMtemp		; 1 cycle
		brne		bit_delay1	; 2 if true, 1 if false
		lsr			temp		; 1  Get next bit
		dec			txbitcnt	; 1  If not all bit sent
		brne		putMIDI0	; 2  send next
		ret			


;********************************************************************************                                          
; GET_SCAN_DN:  ; signal handler for external interrupt int0 (AVR pin 6)                                                      
;   called by falling transition of the keyboard clock signal                                                                 
;   assemble the scancode from keyboard data signal- data valid when clock is low
;  22 cycles for mid-byte data; 20 start and parity bits; 25 for stop bit
;                                       
; (  use Terminal font to see correctly )                                                                                  
;clk---|_|--|_|--|_|--|_|--|_|--|_|--|_|--|_|--|_|---|_|---|_|----------   (---- = logic high +5v)                         
;data-\___/-----\___/-----\___/-----\___/-----\___/--------------------                                                    
;    11   10    9    8    7    6    5    4    3     2     1 bitcount on falledge before dec                                         
;   start    0    1    2    3    4    5    6    7     P   stop   data bit                                                  
;  Bitcnt 3 to 10 is data. start bit = 11, parity bit=bit 2;  stop bit = bit 1                                             
;*********************************************************************************                                         
get_scan_dn:                                                    
		cpi 	bitcount,INITIAL_BITCOUNT 	; falling edge of the FIRST pulse from the keyboard -start bit so do nothing   
		breq	GSD0                                                                                                      
		cpi		bitcount,PARITY_BITCOUNT 	; = 3  ; test for parity bit and stop bit                                      
		brlo	GSD0  						; must use bitcount 3 for compare because branch tests only for lower          
		lsr		scanbits   					; shift data right one bit - data bit 7 gets 0                                 
		sbic	KBport,KBdataPin  			; set scancode bit if keyboard data bit is set                                  
		ori		scanbits,$80 				; if data from kbrd is 1, then set bit 7 only and let other bits unchanged    	                                                                                                   
GSD0:	dec 	bitcount                                                                                                   
		brne	GSD1						; All bits received?                                                           
		ldi		bitcount,INITIAL_BITCOUNT                                                                                  
		set									; set T flag to let main program know a new char is ready                      
		mov		scancode,scanbits 			; scanbits can be used to assemble new char while scancode is being decoded    
GSD1: 	reti
		
		
;  shows program and version information when code data downloaded from an AVR is viewed with a hex editor                                   
ID:		
.db    "****  CPBK21a.msv .105A al nrPbonatdM ya2 ,42 00 4"   
; .db  "****  PCKB12.asm v1.50 Alan Probandt May 24, 2004"


;*******************************************************************************
;    keypress tables      PC keyboard scancode, value returned in 'DecodeValue'
;
;  end of table marked with zero   46 notes (B1 to F#4)-Space-12 Function keys 
;*******************************************************************************
.eseg

unshifted: 
;        a#  b   c   c#  d   d#  e
.db $0d,$0e,$16,$15,$1e,$1d,$26,$24	; 00 - 07  notes A#1 to E1
;    f   f#  g   g#  a   a#  b   c
.db $2d,$2e,$2c,$36,$35,$3d,$3c,$43	; 08 - 0f  notes F1 to C2
;    c#  d   d#  e   f   f#  g   g# 
.db $46,$44,$45,$4d,$54,$55,$5b,$66	; 10 - 17  notes C#2 to G#3
;    a   a#  b   c   c#  d   d#  e
.db $5d,$1c,$12,$1a,$1b,$22,$23,$21	; 18 - 1f  notes A3  to E3
;    f   f#  g   g#  a   a#  b   c
.db $2a,$34,$32,$33,$31,$3b,$3a,$41	; 20 - 27  notes F3  to C4
;    c#  d   d#  e   f   f#   g  sp 
.db $4b,$49,$4c,$4a,$59,$52,$5a,$29, ; C#3 - g3 29=spacebar (offset=$2e) 
;    Fn1 Fn2 Fn3 Fn4 Fn5 Fn6 Fn7 Fn8
.db $05, $06,$04,$0c,$03,$0b,$83,$0a ; 30 - 37  Fn1 Fn2 Fn3 Fn4 Fn5 Fn6 Fn7 Fn8
;   Fn9 F10 F11 F12
.db $01,$09,$78,$07, 0					; 38 - 3f   Fn9 Fn10 Fn11 Fn12


